@@ -1,14 +1,14 @@
 #!/usr/bin/env python
 # vim: set fileencoding=utf-8 :
 
-VERSION = "0.7.7"
-VERDATE = "2013 Oct 18"
+VERSION = "0.7.8"
+VERDATE = "2014 Sep 19"
 
 # Standard library
 import ConfigParser
 import calendar
-from collections import deque
 from copy import copy
+import distutils.sysconfig
 import fnmatch
 from hashlib import md5 as md5
 import math
@@ -16,6 +16,7 @@ import os
 import os.path
 import random
 import re
+import site
 import socket
 from subprocess import call
 import sys
@@ -34,6 +35,7 @@ except:
     iqm_available = False
 
 
+LINE_BUFFERED = False
 LOG_LEVEL = 1                   # 0 == quiet, 1 == normal, 2 == debug
 DISC_SYNC_CNT = 100000          # number of records to hold in memory before
                                 #   flushing to disk
@@ -65,10 +67,10 @@ ipcnts = dict()
 re_cmp = re.compile(r"([a-z]+)(>|<|=|\!=|\!~|\~)([^,]+)")
 if iqm_available:
     re_agg = re.compile(r"(?:(avg|count|dev|iqm|max|min|miqm|sum|var)"
-                         "\(([a-z\*1]+|)\)|([a-z]+))")
+                        "\(([a-z\*1]+|)\)|([a-z]+))")
 else:
     re_agg = re.compile(r"(?:(avg|count|dev|max|min|sum|var)"
-                         "\(([a-z\*1]+|)\)|([a-z]+))")
+                        "\(([a-z\*1]+|)\)|([a-z]+))")
 
 # translate log format string into a column list and regexp
 re_str = r"(\S+)"
@@ -114,16 +116,68 @@ LOG_DIRECTIVES = {
 }
 
 
+def debug(s):
+    if LOG_LEVEL < 2:
+        return
+    sys.stderr.write(s + "\n")
+    if LINE_BUFFERED:
+        sys.stderr.flush()
+
+
 def warn(s):
     if LOG_LEVEL < 1:
         return
     sys.stderr.write(s + "\n")
+    if LINE_BUFFERED:
+        sys.stderr.flush()
 
 
-def cfg_home():
-    if (os.name != "posix"):
-        return sys.prefix
-    return "/etc"
+def find_cfg_file():
+    """Find cfg_file by searching from most accurate/specific to least.
+
+    1. VirtualEnv + /etc/wtop.cfg
+    2. PYTHONUSERBASE + /etc/wtop.cfg
+    3. USER_BASE + /etc/wtop.cfg
+    4. Python Lib + /etc/wtop.cfg
+    5. /etc/wtop.cfg
+    """
+    def test_file(cfg_prefix, cfg_suffix):
+        """Concatenate paths and test if resulting path is a file."""
+        cfg_file = os.path.join(cfg_prefix, cfg_suffix)
+        if os.path.isfile(cfg_file):
+            debug("Using cfg_file: %s" % cfg_file)
+            return cfg_file
+        else:
+            debug("Cfg_file skipped. Cfg_file not found: %s" %
+                  cfg_file)
+            return None
+
+    etc_wtop_path = os.path.join("etc", "wtop.cfg")
+    # VirtualEnv
+    if hasattr(sys, "real_prefix"):
+        cfg_file = test_file(sys.prefix, etc_wtop_path)
+        if cfg_file:
+            return cfg_file
+    # PYTHONUSERBASE
+    user_base = os.environ.get("PYTHONUSERBASE")
+    if user_base:
+        cfg_file = test_file(user_base, etc_wtop_path)
+        if cfg_file:
+            return cfg_file
+    # USER_BASE
+    cfg_file = test_file(site.USER_BASE, etc_wtop_path)
+    if cfg_file:
+        return cfg_file
+    # Distutils Python Lib
+    cfg_file = test_file(distutils.sysconfig.get_python_lib(), etc_wtop_path)
+    if cfg_file:
+        return cfg_file
+    # /etc/wtop.cfg (for backwards compatibility
+    cfg_file = test_file("/", "etc/wtop.cfg")
+    if cfg_file:
+        return cfg_file
+    # fail.
+    raise Exception("Cfg_file could not be found")
 
 
 # yes, this is ugly.
@@ -132,11 +186,7 @@ def configure(cfg_file=None):
     global LOG_FORMAT, LOG_PATTERN, LOG_ROOT, MAX_REQUEST_TIME, MIN_RPS
     global config, re_classes, re_generic, re_robots
 
-    if not cfg_file:
-        cfg_file = os.path.join(cfg_home(), "wtop.cfg")
-
-    if not os.path.exists(cfg_file):
-        raise Exception("can't read config file: '%s'" % cfg_file)
+    cfg_file = find_cfg_file()
 
     config = ConfigParser.ConfigParser()
     config.read(cfg_file)
@@ -827,6 +877,8 @@ def apache_top_mode(reqs):
 def print_mode(reqs, fields):
     for r in reqs:
         print "\t".join([str(r[k]) for k in fields])
+        if LINE_BUFFERED:
+            sys.stdout.flush()
 
 
 # compact ids for a dict, given a list of keys to use as the unique identifier
@@ -902,12 +954,12 @@ def calculate_aggregates(reqs, agg_fields, group_by, order_by=None, limit=0,
             return numerator / denominator
 
     def agg_iqm(i, r, field, table, key):
-        key="%s-%s" % (key, i)
+        key = "%s-%s" % (key, i)
         diqm(key, r[field])
         return (0, 0)
 
     def agg_miqm(i, r, field, table, key):
-        key="%s-%s" % (key, i)
+        key = "%s-%s" % (key, i)
         miqm(key, r[field])
         return (0, 0)
 
@@ -1028,6 +1080,8 @@ def calculate_aggregates(reqs, agg_fields, group_by, order_by=None, limit=0,
 def agg_mode(rows, fmt):
     for row in rows:
         print fmt % tuple(row[1:])
+        if LINE_BUFFERED:
+            sys.stdout.flush()
 
 
 ## experimental RRDtool mode for generating timeseries graphs
@@ -1157,8 +1211,9 @@ def rrd_mode(reqs, step=5, msec_max=2000, hist_steps=10, hist_scale=100,
                     continue
                 v = stats[k]
 
-                if do_hist:
-                    histogram = normalize(v["hist"], v["count"][0], hist_scale)
+# Not used. Strange. Disable for now.
+#               if do_hist:
+#                   histogram = normalize(v["hist"], v["count"][0], hist_scale)
 
                 call(["rrdtool", "update", "%s.rrd" % k,
                       "%d:%s:%s:%s:%s:%s" %
